% Marco San Biagio      Version 1.00
% Copyright 2013 Marco San Biagio.  [Marco.SanBiagio-at-iit.it]
% Please email me if you have questions.
%
% [1] M. San Biagio, M. Crocco, M. Cristani, S. Martelli and V. Murino
% Heterogeneous Auto-Similarities of Characteristics (HASC): Exploiting Relational Information for Classification
% ICCV 2013

% -------------------------------------------- %
%                                         Main %
% -------------------------------------------- %

clc
clear
warning off;

addpath(genpath('./my_utils/'));

disp('% ------------------------------------------ %');
disp('%                           Starting Program %');
disp('% ------------------------------------------ %');

format shortg;
c = clock;

% Number of classes
opt.NofClasses      = 102;
% Number of training examples
opt.NofTraining     = 15;
% Block X dimension
opt.BlocksX         = 8;
% Block Y dimension
opt.BlocksY         = opt.BlocksX;
% % Overlap
opt.PercOverlap     = 0.5;
% Number of pixels for x and y gradient
opt.FeatsBorder     = 2;
% Number of used features
opt.NofFeatures     = 15;
% Number of covariance + emi elements
opt.HascDim         = opt.NofFeatures*(opt.NofFeatures+1);
% Caltech101 dataset directory
opt.Directory       = './Caltech101 Dataset';
opt.autoDownloadData = 1;
% Caltech101 dataset Normalization directory
opt.Norm            = './Norm/';
% Flag save variables
opt.Save            = 0;
% Number of BINs
opt.Bin             = 28;
% Output directory
opt.Experiments     = ['./Outputs/Caltech101/' date '-' num2str(c(4)) '.' num2str(c(5)) '.' ...
    num2str(round(c(6))) '-Blocks8'];
% C value for SVM
opt.bestc           = 10;
opt.bestc2          = 1;
opt.bestc3          = 1;

if opt.Save
    % Check Experiment Directory
    if(isdir(opt.Experiments)==0)
        mkdir(opt.Experiments);
    end
    % Check Normalization Directory
    if(isdir(opt.Norm)==0)
        mkdir(opt.Norm);
    end
end

clear c;

% --------------------------------------------------------------------
%                                            Download Caltech-101 data
% --------------------------------------------------------------------

disp('% ------------------------------------------ %');
disp('%                    Downloading Caltech-101 %');

if ~exist(opt.Directory , 'dir') || ...
   (~exist(fullfile(opt.Directory , 'airplanes'),'dir') && ...
    ~exist(fullfile(opt.Directory , '101_ObjectCategories', 'airplanes')))
  if ~opt.autoDownloadData
    error(...
      ['Caltech-101 data not found. ' ...
       'Set opt.autoDownloadData=1 to download the required data.']) ;
  end
  vl_xmkdir(opt.Directory) ;
  calUrl = ['http://www.vision.caltech.edu/Image_Datasets/' ...
    'Caltech101/101_ObjectCategories.tar.gz'] ;
 
  untar(calUrl, opt.Directory) ;
end

if ~exist(fullfile(opt.Directory, 'airplanes'),'dir')
  opt.Directory = fullfile(opt.Directory, '101_ObjectCategories') ;
end

clear calUrl

disp('% ------------------------------------------ %');

% -------------------------------------------- %
%                             Image Selections %
% -------------------------------------------- %

% dirClasses = dir(opt.Directory);
% 
% isub = [dirClasses(:).isdir];
% nameFolds = {dirClasses(isub).name}';
% 
% dirClasses(ismember(nameFolds,{'.','..'})) = [];
% 
% clear nameFolds isub

% -------------------------------------------- %
%                          Features Extraction %
% -------------------------------------------- %

disp('% ------------------------------------------ %');
disp('%                        Features Extraction %');

fileload = './Split/SplitUCSD.mat';
load(fileload, 'test_fnames', 'train_fnames', 'test_labels', 'train_labels', 'labels_train', 'labels_test');

StructImagesXClasses = cell(1, opt.NofTraining*opt.NofClasses);
for i = 1 : opt.NofTraining*opt.NofClasses
    
    if(mod(i,500)==0), disp(['% -> Elaborating Image ' num2str(i) '/' num2str(opt.NofTraining*opt.NofClasses)]); end;
    
    folder_image_name = train_fnames(i,:);
    img = imread([opt.Directory '/' folder_image_name]);
    img = uint8(img);
    
    img = imresize(img, [150 150]);
    img_feat = msb_GetFeatures(img, opt.FeatsBorder, opt.NofFeatures, opt.BlocksX, opt.BlocksY, opt.PercOverlap);
    
    StructImagesXClasses{1,i} = img_feat;
    
end

clear fileload i img folder_image_name img_feat;

disp('% ------------------------------------------ %');

% -------------------------------------------- %
%              Mean and STD Feature Estimation %
% -------------------------------------------- %

disp('% ------------------------------------------ %');
disp('%              Loading Mean and STD features %');

name1 =   [opt.Norm 'MeanStdFeatures.mat'];
load(name1);
clear name1;

disp('% ------------------------------------------ %');

% -------------------------------------------- %
%                              HASC Estimation %
% -------------------------------------------- %

disp('% ------------------------------------------ %');
disp('%                            HASC Estimation %');

min_rap_training = zeros(size(StructImagesXClasses,2),opt.HascDim*opt.BlocksX*opt.BlocksY);
min_rap_trainingEMI = zeros(size(StructImagesXClasses,2),(opt.HascDim*opt.BlocksX*opt.BlocksY)/2);
min_rap_trainingCOV = zeros(size(StructImagesXClasses,2),(opt.HascDim*opt.BlocksX*opt.BlocksY)/2);

e = 1;

if opt.Save
    name1 =   [opt.Experiments '/MinRapTraining_BIN' num2str(opt.Bin) '_Patch' num2str(opt.BlocksX)];
    name2 =   [opt.Experiments '/MinRapTraining_scaled_BIN' num2str(opt.Bin) '_Patch' num2str(opt.BlocksX)];
end

for i = 1 : size(StructImagesXClasses,2)
    tmp_EMI = [];
    tmp_COV = [];
    
    if(mod(e,500)==0), disp(['% -> Processing image ' num2str(e) '/' num2str(opt.NofTraining*opt.NofClasses)]); end;
    
    [DimBlkY,DimBlkX] = size(StructImagesXClasses{1,i}{opt.NofFeatures});
    
    RealOverlapX = floor(DimBlkX*opt.PercOverlap);
    RealOverlapY = floor(DimBlkY*opt.PercOverlap);
    
    tmp_image = cell(opt.NofFeatures-2,1);
    for nof = 1 : opt.NofFeatures-2
        tmp_image{nof} = StructImagesXClasses{1,i}{nof};
    end
    
    for blk_y = 1 : opt.BlocksY
        
        for blk_x = 1 : opt.BlocksX
            
            block = zeros(DimBlkY,DimBlkX,opt.NofFeatures);
            for nof = 1 : opt.NofFeatures-2
                block(:,:,nof) = tmp_image{nof}( ((blk_y-1)*RealOverlapY+1):((blk_y-1)*RealOverlapY+DimBlkY),((blk_x-1)*RealOverlapX+1):((blk_x-1)*RealOverlapX+DimBlkX) );
                block(:,:,nof) = (block(:,:,nof) - mean_vector(1,nof))./(std_vector(1,nof)+eps);
            end
            
            block(:,:,opt.NofFeatures-1) = StructImagesXClasses{1,i}{opt.NofFeatures-1};
            block(:,:,opt.NofFeatures) = StructImagesXClasses{1,i}{opt.NofFeatures};
            
            [EMI_vec, COV_vec] = msb_EmiCovMex( block, opt.Bin, opt.Norm);
            
            tmp_EMI = [tmp_EMI EMI_vec];
            tmp_COV = [tmp_COV COV_vec];
            
        end
        
    end
    
    min_rap_training(e,:) = [tmp_EMI tmp_COV];
    min_rap_trainingEMI(e,:) = tmp_EMI;
    min_rap_trainingCOV(e,:) = tmp_COV;
    e = e + 1;
    
end

if opt.Save
    save(name1, 'min_rap_training','labels_train', '-v7.3');
end

disp('% ------------------------------------------ %');

clear e tmp_COV tmp_EMI tmp_image blk_x blk_y nof i DimBlkX DimBlkY mean_vector ...
    std_vector RealOverlapX RealOverlapY block max_vector min_vector EMI_vec COV_vec ...
    StructImagesXClasses name1;

% -------------------------------------------- %
%                               Training Phase %
% -------------------------------------------- %

disp('% ------------------------------------------ %');
disp('%                             Training Phase %');

if opt.Save
    name1 =   [opt.Experiments '/ModelsTraining_BIN' num2str(opt.Bin) '_Patch' num2str(opt.BlocksX) '.mat'];
end

scale_factorEMI = max(abs(min_rap_trainingEMI))+eps;
numEMI = size(min_rap_trainingEMI,1);
matrix_scaledEMI = zeros(size(min_rap_trainingEMI));
for i = 1: numEMI
    matrix_scaledEMI(i,:) = min_rap_trainingEMI(i,:)./scale_factorEMI;
end

scale_factorCOV = max(abs(min_rap_trainingCOV))+eps;
numCOV = size(min_rap_trainingCOV,1);
matrix_scaledCOV = zeros(size(min_rap_trainingCOV));
for i = 1: numCOV
    matrix_scaledCOV(i,:) = min_rap_trainingCOV(i,:)./scale_factorCOV;
end

scale_factor = max(abs(min_rap_training))+eps;
num = size(min_rap_training,1);
matrix_scaled = zeros(size(min_rap_training));
for i = 1: num
    matrix_scaled(i,:) = min_rap_training(i,:)./scale_factor;
end

if opt.Save
    save(name2, 'matrix_scaled','labels_train', '-v7.3');
end

tic
model = train( double(labels_train) , sparse(matrix_scaled) , ['-c ', num2str(opt.bestc), '-e 0.001 -b 1']);
modelEMI = train( double(labels_train) , sparse(matrix_scaledEMI) , ['-c ', num2str(opt.bestc2), '-e 0.001 -b 1']);
modelCOV = train( double(labels_train) , sparse(matrix_scaledCOV) , ['-c ', num2str(opt.bestc3), '-e 0.001 -b 1']);
toc

if opt.Save
    bestc =  opt.bestc;
    save(name1, 'model', 'scale_factor', 'bestc', '-v7.3');
end

disp('% ------------------------------------------ %');

clear i numCOV numEMI num labels_train matrix_scaled matrix_scaledEMI ...
    matrix_scaledCOV min_rap_training min_rap_trainingCOV min_rap_trainingEMI ...
    name1;

% -------------------------------------------- %
%                                Testing Phase %
% -------------------------------------------- %

disp('% ------------------------------------------ %');
disp('%                              Testing Phase %');

name2 =   [opt.Norm '/MeanStdFeatures.mat'];
load(name2);

min_rap_testing = zeros(size(test_fnames,1), opt.HascDim*opt.BlocksX*opt.BlocksY);
min_rap_testingEMI = zeros(size(test_fnames,1), (opt.HascDim*opt.BlocksX*opt.BlocksY)/2);
min_rap_testingCOV = zeros(size(test_fnames,1), (opt.HascDim*opt.BlocksX*opt.BlocksY)/2);
TestingExamples = size(test_fnames,1);
disp(['-> Total of images ' num2str(TestingExamples)]);
e = 1;

for i = 1 : TestingExamples
    
    if(mod(e,500)==0), disp(['% -> Processing image ' num2str(e) '/' num2str(TestingExamples)]); end;
    
    tmp_EMI = [];
    tmp_COV = [];
    
    folder_image_name = test_fnames(i,:);
    img = imread([opt.Directory '/' folder_image_name]);
    img = uint8(img);
    img = imresize(img, [150 150]);
    img_feat = msb_GetFeatures(img, opt.FeatsBorder, opt.NofFeatures, opt.BlocksX, opt.BlocksY, opt.PercOverlap);
    
    [DimBlkY,DimBlkX] = size(img_feat{opt.NofFeatures});
    
    RealOverlapX = floor(DimBlkX*opt.PercOverlap);
    RealOverlapY = floor(DimBlkY*opt.PercOverlap);
    
    for blk_y = 1 : opt.BlocksY
        
        for blk_x = 1 : opt.BlocksX
            
            block = zeros(DimBlkY,DimBlkX,opt.NofFeatures);
            
            for nof = 1 : opt.NofFeatures-2
                block(:,:,nof) = img_feat{nof}( ((blk_y-1)*RealOverlapY+1):((blk_y-1)*RealOverlapY+DimBlkY),((blk_x-1)*RealOverlapX+1):((blk_x-1)*RealOverlapX+DimBlkX) );
                block(:,:,nof) = (block(:,:,nof) - mean_vector(1,nof))./(std_vector(1,nof)+eps);
            end
            block(:,:,opt.NofFeatures-1) = img_feat{opt.NofFeatures-1};
            block(:,:,opt.NofFeatures) = img_feat{opt.NofFeatures};
            
            [EMI_vec, COV_vec] = msb_EmiCovMex( block, opt.Bin, opt.Norm);

            tmp_EMI = [tmp_EMI EMI_vec];
            tmp_COV = [tmp_COV COV_vec];
            
        end
        
    end
    
    min_rap_testing(e,:) = [tmp_EMI tmp_COV];
    min_rap_testingEMI(e,:) = tmp_EMI;
    min_rap_testingCOV(e,:) = tmp_COV;
    e = e + 1;
    
end

clear e tmp_COV tmp_EMI blk_x blk_y nof i DimBlkX DimBlkY img img_feat mean_vector ...
    std_vector RealOverlapX RealOverlapY block max_vector min_vector EMI_vec COV_vec ...
    name1 TestingExamples folder_image_name;

if opt.Save
    name1 =   [opt.Experiments '/MinRapTesting_BIN' num2str(opt.Bin) '_Patch' num2str(opt.BlocksX) '.mat'];
    save(name1, 'min_rap_testing','labels_test', '-v7.3');
end

numEMI = size(min_rap_testingEMI,1);
matrix_scaledEMI = zeros(size(min_rap_testingEMI));
for nu = 1: numEMI
    matrix_scaledEMI(nu,:) = min_rap_testingEMI(nu,:)./scale_factorEMI;
end

numCOV = size(min_rap_testingCOV,1);
matrix_scaledCOV = zeros(size(min_rap_testingCOV));
for nu = 1: numCOV
    matrix_scaledCOV(nu,:) = min_rap_testingCOV(nu,:)./scale_factorCOV;
end

num = size(min_rap_testing,1);
matrix_scaled = zeros(size(min_rap_testing));
for nu = 1: num
    matrix_scaled(nu,:) = min_rap_testing(nu,:)./scale_factor;
end

if opt.Save
    name2 =   [opt.Experiments '/MinRapTesting_scaled_BIN' num2str(opt.Bin) '_Patch' num2str(opt.BlocksX) '.mat'];
    save(name2, 'matrix_scaled','labels_test', '-v7.3');
end

[label, accuracy_test, conf] = predict(labels_test, sparse(matrix_scaled), model);
[labelEMI, accuracy_testEMI, confEMI] = predict(labels_test, sparse(matrix_scaledEMI), modelEMI);
[labelCOV, accuracy_testCOV, confCOV] = predict(labels_test, sparse(matrix_scaledCOV), modelCOV);

disp('% ------------------------------------------ %');


clear dirClasses labels_test min_rap_testing min_rap_testingCOV min_rap_testingEMI ...
    matrix_scaled matrix_scaledCOV matrix_scaledEMI model modelCOV modelEMI ...
    name2 num numCOV numEMI opt scale_factor scale_factorCOV scale_factorEMI ...
    test_fnames test_labels train_fnames train_labels nu
    

% -------------------------------------------- %
%                            Finishing Program %
% -------------------------------------------- %

disp('% ------------------------------------------ %');
disp('%                          Finishing Program %');
disp('% ------------------------------------------ %');
