function [ Feat ] = msb_GetFeatures(Data, NoPixelsBorder, Nofeat, NoBlkX, NoBlkY, PercOverlap)

    % Marco San Biagio      Version 1.00
    % Copyright 2013 Marco San Biagio.  [Marco.SanBiagio-at-iit.it]
    % Please email me if you have questions.
    %
    % INPUTS
    % Data                 - RGB or GRAY image (MxNx3 or MxN)
    % NoPixelsBorder       - set to 2 because of the gradient computation
    % Nofeat               - number of features
    % NoBlkX               - number of blocks along the X direction
    % NoBlkY               - number of blocks along the Y direction
    % PercOverlap          - overlap between each block
    %
    % OUTPUT
    % Feat                 - 3D matrix representing the features extracted from the image (MxNxd)
    %
    % [1] M. San Biagio, M. Crocco, M. Cristani, S. Martelli and V. Murino
    % Heterogeneous Auto-Similarities of Characteristics (HASC): Exploiting Relational Information for Classification
    % ICCV 2013


    % FEATURE'S PARAMETERS
    % Size Image
    si = size(Data);
    if length(si) > 2
        img = rgb2gray(Data);
    else
        img = Data;
    end
    [hf,wf] = size(img);

    % Image Size without feature borders
    HFwoFB = hf-2*NoPixelsBorder;
    WFwoFB = wf-2*NoPixelsBorder;

    % Creating X and Y Position Matrices  
    DimBlkX = floor(WFwoFB/((1-PercOverlap)*(NoBlkX-1)+1));
    DimBlkY = floor(HFwoFB/((1-PercOverlap)*(NoBlkY-1)+1));

    RealOverlapX = floor(DimBlkX*(1-PercOverlap));
    RealOverlapY = floor(DimBlkY*(1-PercOverlap));

    cutimageX = WFwoFB - (RealOverlapX*(NoBlkX-1)+DimBlkX);
    cutimageY = HFwoFB - (RealOverlapY*(NoBlkY-1)+DimBlkY);

    if cutimageX ~= 0
        cut_x1 = floor(cutimageX/2);
        cut_x2 = ceil(cutimageX/2);
    else
        cut_x1 = 0;
        cut_x2 = 0;
    end

    if cutimageY ~= 0
        cut_y1 = floor(cutimageY/2);
        cut_y2 = ceil(cutimageY/2);
    else
        cut_y1 = 0;
        cut_y2 = 0;
    end

    Feat = cell(Nofeat,1);
    [pos_featurex,pos_featurey] = meshgrid(linspace(-1,1,DimBlkX),linspace(-1,1,DimBlkY));

    % Parameters
    hx = [-1,0,1];
    hy = -hx';
    K = 1.646760;

    % Padding for Gradient X and Y
    ln_to_pad  = NoPixelsBorder;
    col_to_pad = NoPixelsBorder;

    % First order Gradient (x->Horizontal y->Vertical)
    Sobelx_Sw = imfilter(double(img),hx);
    Sobely_Sw = imfilter(double(img),hy);
    ml_pad_1 = Sobelx_Sw;
    ml_pad_2 = Sobely_Sw;
    Sobelhh_Sw = imfilter(double(ml_pad_1),hx);
    Sobelvv_Sw = imfilter(double(ml_pad_2),hy);
    % Second order Gradient
    ml_pad_3 = Sobelhh_Sw;
    ml_pad_4 = Sobelvv_Sw;
    % Magnitude
    ml_pad_5 = K*sqrt(ml_pad_1.^2 + ml_pad_2.^2);
    % LBP
    ml_pad_6 = double(GetLBP(img));

    % Creating Feature Structure
    Feat{1} = abs(ml_pad_1(ln_to_pad+1+cut_y1:hf-ln_to_pad-cut_y2,col_to_pad+1+cut_x1:wf-col_to_pad-cut_x2));
    Feat{2} = abs(ml_pad_2(ln_to_pad+1+cut_y1:hf-ln_to_pad-cut_y2,col_to_pad+1+cut_x1:wf-col_to_pad-cut_x2));
    Feat{3} = abs(ml_pad_3(ln_to_pad+1+cut_y1:hf-ln_to_pad-cut_y2,col_to_pad+1+cut_x1:wf-col_to_pad-cut_x2));
    Feat{4} = abs(ml_pad_4(ln_to_pad+1+cut_y1:hf-ln_to_pad-cut_y2,col_to_pad+1+cut_x1:wf-col_to_pad-cut_x2));
    Feat{5} = ml_pad_5(ln_to_pad+1+cut_y1:hf-ln_to_pad-cut_y2,col_to_pad+1+cut_x1:wf-col_to_pad-cut_x2);
    Feat{6} = ml_pad_6(ln_to_pad+1-1+cut_y1:hf-ln_to_pad-1-cut_y2,col_to_pad+1-1+cut_x1:wf-col_to_pad-1-cut_x2);

    if length(si) == 2

        Feat{7} = double(img(ln_to_pad+1+cut_y1:hf-ln_to_pad-cut_y2,col_to_pad+1+cut_x1:wf-col_to_pad-cut_x2,1));
        Data(:,:,1) = img;
        Data(:,:,2) = img;
        Data(:,:,3) = img;

        Feat{8} = Feat{7};
        Feat{9} = Feat{7};

    else

        Feat{7} = double(Data(ln_to_pad+1+cut_y1:hf-ln_to_pad-cut_y2,col_to_pad+1+cut_x1:wf-col_to_pad-cut_x2,1));
        Feat{8} = double(Data(ln_to_pad+1+cut_y1:hf-ln_to_pad-cut_y2,col_to_pad+1+cut_x1:wf-col_to_pad-cut_x2,2));
        Feat{9} = double(Data(ln_to_pad+1+cut_y1:hf-ln_to_pad-cut_y2,col_to_pad+1+cut_x1:wf-col_to_pad-cut_x2,3));

    end

    cielab = RGB2Lab(Data);

    Feat{10} = cielab(ln_to_pad+1+cut_y1:hf-ln_to_pad-cut_y2,col_to_pad+1+cut_x1:wf-col_to_pad-cut_x2,1);
    Feat{11} = cielab(ln_to_pad+1+cut_y1:hf-ln_to_pad-cut_y2,col_to_pad+1+cut_x1:wf-col_to_pad-cut_x2,2);
    Feat{12} = cielab(ln_to_pad+1+cut_y1:hf-ln_to_pad-cut_y2,col_to_pad+1+cut_x1:wf-col_to_pad-cut_x2,3);

    img_filtered = medfilt2(img, [5,5]);
    Feat{13} = double(img_filtered(ln_to_pad+1+cut_y1:hf-ln_to_pad-cut_y2,col_to_pad+1+cut_x1:wf-col_to_pad-cut_x2));

    Feat{14} = pos_featurex;
    Feat{15} = pos_featurey;

end